from tests.core.helpers import pagination_response

from hamcrest import *
from six.moves import xrange
from trakt import Trakt
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/sync/history/movies',
        callback=pagination_response(
            'fixtures/sync/history/movies.json',
            authenticated=True
        )
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        history = Trakt['sync/history'].movies(pagination=True, per_page=5)

    # Ensure collection is valid
    assert_that(history, not_none())

    # Resolve all pages
    items = list(history)

    # Ensure all items have been returned
    assert_that(items, has_length(25))

    # Verify item identifiers
    assert_that(
        [item.id for item in items],
        equal_to(list(xrange(1, 26)))
    )



from trakt import Trakt

import json
import os
import six


def authenticate():
    authorization = os.environ.get('AUTHORIZATION')

    if authorization:
        return json.loads(authorization)

    print('Navigate to: %s' % Trakt['oauth'].authorize_url('urn:ietf:wg:oauth:2.0:oob'))

    code = six.moves.input('Authorization code:')
    if not code:
        exit(1)

    authorization = Trakt['oauth'].token(code, 'urn:ietf:wg:oauth:2.0:oob')
    if not authorization:
        exit(1)

    print('Authorization: %r' % authorization)
    return authorization



from tests.core.helpers import pagination_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
from trakt.objects import Movie, Show, Season, Episode
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/sync/watchlist',
        callback=pagination_response(
            'fixtures/sync/watchlist.json',
            authenticated=True
        )
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        watchlist = Trakt['sync/watchlist'].get(pagination=True, per_page=3)

    # Ensure collection is valid
    assert_that(watchlist, not_none())

    # Resolve all pages
    items = list(watchlist)

    # Validate items
    assert_that(items, contains(
        # TRON: Legacy (2010)
        all_of(
            instance_of(Movie),
            has_properties({
                'pk': ('imdb', 'tt1104001'),
                'title': 'TRON: Legacy',
                'year': 2010,

                # Timestamps
                'listed_at': datetime(2014, 9, 1, 9, 10, 11, tzinfo=tzutc()),

                # Keys
                'keys': [
                    ('imdb',  'tt1104001'),
                    ('tmdb',  '20526'),
                    ('slug',  'tron-legacy-2010'),
                    ('trakt', '1')
                ]
            })
        ),

        # Breaking Bad (2008)
        all_of(
            instance_of(Show),
            has_properties({
                'pk': ('tvdb', '81189'),
                'title': 'Breaking Bad',
                'year': 2008,

                # Timestamps
                'listed_at': datetime(2014, 9, 1, 9, 10, 11, tzinfo=tzutc()),

                # Keys
                'keys':[
                    ('tvdb', '81189'),
                    ('tmdb', '1396'),
                    ('imdb', 'tt0903747'),
                    ('tvrage', '18164'),
                    ('slug', 'breaking-bad'),
                    ('trakt', '1')
                ]
            })
        ),

        # Breaking Bad (2008) - S03
        all_of(
            instance_of(Season),
            has_properties({
                'pk': 3,

                # Timestamps
                'listed_at': datetime(2014, 9, 1, 9, 10, 11, tzinfo=tzutc()),

                # Keys
                'keys': [
                    3,
                    ('tvdb', '171641'),
                    ('tmdb', '3575')
                ]
            })
        ),

        # Breaking Bad (2008) - S04E01
        all_of(
            instance_of(Episode),
            has_properties({
                'pk': (4, 1),
                'title': 'Box Cutter',

                # Timestamps
                'listed_at': datetime(2014, 9, 1, 9, 10, 11, tzinfo=tzutc()),

                # Keys
                'keys': [
                    (4, 1),
                    ('tvdb', '2639411'),
                    ('tmdb', '62118'),
                    ('imdb', 'tt1683084'),
                    ('trakt', '49')
                ]
            })
        )
    ))



from datetime import datetime

from helpers import authenticate

from trakt import Trakt
import os


if __name__ == '__main__':
    # Configure
    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID'),
        secret=os.environ.get('CLIENT_SECRET')
    )

    # Authenticate
    Trakt.configuration.defaults.oauth.from_response(
        authenticate()
    )

    now = datetime.utcnow()

    # Retrieve 10 history records (most recent)
    for item in Trakt['sync/history'].get(per_page=10):
        print(' - %-120s (watched_at: %r)' % (
            repr(item),
            item.watched_at.strftime('%Y-%m-%d %H:%M:%S')
        ))

    print('=' * 160)

    # Retrieve history records for "Family Guy"
    for item in Trakt['sync/history'].shows('1425', pagination=True, per_page=25):
        print(' - %-120s (watched_at: %r)' % (
            repr(item),
            item.watched_at.strftime('%Y-%m-%d %H:%M:%S')
        ))

    print('=' * 160)

    # Retrieve history records for this year
    for item in Trakt['sync/history'].get(pagination=True, per_page=25, start_at=datetime(now.year, 1, 1)):
        print(' - %-120s (watched_at: %r)' % (
            repr(item),
            item.watched_at.strftime('%Y-%m-%d %H:%M:%S')
        ))

    print('=' * 160)

    # Retrieve all history records
    for item in Trakt['sync/history'].get(pagination=True, per_page=25):
        print(' - %-120s (watched_at: %r)' % (
            repr(item),
            item.watched_at.strftime('%Y-%m-%d %H:%M:%S')
        ))



from tests.core.helpers import assert_url, authenticated_response

from trakt import Trakt
import calendar
import datetime
import json
import pytest
import responses


def test_authorize_url():
    Trakt.site_url = 'http://mock'

    with Trakt.configuration.client('mock', 'mock'):
        assert_url(Trakt['oauth'].authorize_url('urn:ietf:wg:oauth:2.0:oob'), '/oauth/authorize', {
            'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
            'response_type': 'code',
            'client_id': 'mock'
        })

        assert_url(Trakt['oauth'].authorize_url('urn:ietf:wg:oauth:2.0:oob', state='state', username='username'), '/oauth/authorize', {
            'username': 'username',
            'state': 'state',

            'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
            'response_type': 'code',
            'client_id': 'mock'
        })

    with pytest.raises(ValueError):
        Trakt['oauth'].authorize_url('urn:ietf:wg:oauth:2.0:oob')


def test_pin_url():
    with Trakt.configuration.app(id=1234):
        assert_url(Trakt['oauth'].pin_url(), '/pin/1234')

    with pytest.raises(ValueError):
        Trakt['oauth'].pin_url()


@responses.activate
def test_token():
    expected_request = {
        'client_secret': 'mock-client_secret',
        'code': 'ABCD1234',
        'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
        'client_id': 'mock-client_id',
        'grant_type': 'authorization_code'
    }

    def callback(request):
        data = json.loads(request.body)

        if data != expected_request:
            return 403, {}, ''

        return 200, {}, json.dumps({
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        })

    responses.add_callback(
            responses.POST, 'http://api.mock/oauth/token',
            callback=callback,
            content_type='application/json'
    )

    Trakt.base_url = 'http://api.mock'

    # Validate `token_exchange` request/response
    with Trakt.configuration.client('mock-client_id', 'mock-client_secret'):
        assert Trakt['oauth'].token('ABCD1234', 'urn:ietf:wg:oauth:2.0:oob') == {
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        }

    # Ensure `token_exchange` raises a `ValueError` on incorrect configuration
    with pytest.raises(ValueError):
        assert Trakt['oauth'].token('ABCD1234', 'urn:ietf:wg:oauth:2.0:oob')


@responses.activate
def test_token_exchange():
    expected_request = {
        'client_secret': 'mock-client_secret',
        'code': 'ABCD1234',
        'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
        'client_id': 'mock-client_id',
        'grant_type': 'authorization_code'
    }

    def callback(request):
        data = json.loads(request.body)

        if data != expected_request:
            return 403, {}, ''

        return 200, {}, json.dumps({
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        })

    responses.add_callback(
        responses.POST, 'http://api.mock/oauth/token',
        callback=callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://api.mock'

    # Validate `token_exchange` request/response
    with Trakt.configuration.client('mock-client_id', 'mock-client_secret'):
        assert Trakt['oauth'].token_exchange('ABCD1234', 'urn:ietf:wg:oauth:2.0:oob') == {
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        }

    # Ensure `token_exchange` raises a `ValueError` on incorrect configuration
    with pytest.raises(ValueError):
        assert Trakt['oauth'].token_exchange('ABCD1234', 'urn:ietf:wg:oauth:2.0:oob')


@responses.activate
def test_token_refresh():
    expected_request = {
        'client_secret': 'mock-client_secret',
        'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
        'client_id': 'mock-client_id',
        'refresh_token': 'mock-refresh_token',
        'grant_type': 'refresh_token'
    }

    def callback(request):
        data = json.loads(request.body)

        if data != expected_request:
            return 403, {}, ''

        return 200, {}, json.dumps({
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        })

    responses.add_callback(
        responses.POST, 'http://api.mock/oauth/token',
        callback=callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://api.mock'

    # Validate `token_exchange` request/response
    with Trakt.configuration.client('mock-client_id', 'mock-client_secret'):
        assert Trakt['oauth'].token_refresh('mock-refresh_token', 'urn:ietf:wg:oauth:2.0:oob') == {
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        }

    # Ensure `token_exchange` raises a `ValueError` on incorrect configuration
    with pytest.raises(ValueError):
        assert Trakt['oauth'].token_refresh('mock-refresh_token', 'urn:ietf:wg:oauth:2.0:oob')


@responses.activate
def test_request():
    responses.add_callback(
        responses.GET, 'http://mock/sync/collection/movies',
        callback=authenticated_response('fixtures/sync/collection/movies.json'),
        content_type='application/json'
    )

    def callback(request):
        return 200, {}, json.dumps({
            "access_token": "mock",
            "token_type": "bearer",
            "created_at": calendar.timegm(datetime.datetime.utcnow().utctimetuple()),
            "expires_in": 7 * 24 * 60 * 60,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        })

    responses.add_callback(
        responses.POST, 'http://mock/oauth/token',
        callback=callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    # Mock authorization
    authorization = {
        "access_token": "mock",
        "token_type": "bearer",
        "created_at": calendar.timegm(datetime.datetime.utcnow().utctimetuple()),
        "expires_in": 7 * 24 * 60 * 60,
        "refresh_token": "mock-refresh_token",
        "scope": "public"
    }

    # Test valid token
    with Trakt.configuration.oauth.from_response(authorization):
        assert Trakt['sync/collection'].movies() is not None

    # Test expired token
    authorization['expires_in'] = 0

    with Trakt.configuration.oauth.from_response(authorization):
        assert Trakt['sync/collection'].movies() is None

    # Test token refreshing
    with Trakt.configuration\
            .client('mock', 'mock')\
            .oauth.from_response(authorization, refresh=True):
        assert Trakt['sync/collection'].movies() is not None



from tests.core.helpers import read

from trakt import Trakt
import pytest
import responses


@responses.activate
def test_likes():
    responses.add(
        responses.GET, 'http://mock/users/likes',
        body=read('fixtures/users/likes.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    likes = Trakt['users'].likes()
    assert likes is not None

    likes = list(likes)
    assert len(likes) is 3

    assert likes[0].keys == [
        ('trakt', 1519)
    ]

    assert likes[1].keys == [
        ('trakt', '1238362'),
        ('slug', 'star-wars-machete')
    ]

    assert likes[2].keys == [
        ('trakt', '840781'),
        ('slug', 'star-wars-timeline')
    ]


@responses.activate
def test_likes_invalid_response():
    responses.add(
        responses.GET, 'http://mock/users/likes',
        body='Not Found', status=404,
        content_type='text/html'
    )

    Trakt.base_url = 'http://mock'

    likes = Trakt['users'].likes()
    assert likes is not None

    likes = list(likes)
    assert len(likes) is 0


def test_likes_invalid_type():
    Trakt.base_url = 'http://mock'

    with pytest.raises(ValueError):
        likes = Trakt['users'].likes('invalid')
        assert likes is not None

        likes = list(likes)



from tests.core.helpers import read

from six.moves.urllib_parse import parse_qsl, urlparse
from trakt import Trakt
import responses


@responses.activate
def test_movie():
    responses.add(
        responses.GET, 'http://mock/movies/tron-legacy-2010',
        body=read('fixtures/movies/tron-legacy-2010.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    movie = Trakt['movies'].get('tron-legacy-2010')

    assert movie.title == 'TRON: Legacy'
    assert movie.year == 2010

    assert movie.pk == ('imdb', 'tt1104001')

    assert movie.keys == [
        ('imdb', 'tt1104001'),
        ('tmdb', '20526'),
        ('slug', 'tron-legacy-2010'),
        ('trakt', '12601')
    ]


@responses.activate
def test_movie_not_found():
    responses.add(
        responses.GET, 'http://mock/movies/tron-fake-2010',
        body='Not Found', status=404,
        content_type='text/html'
    )

    Trakt.base_url = 'http://mock'

    movie = Trakt['movies'].get('tron-fake-2010')
    assert movie is None


@responses.activate
def test_show():
    responses.add(
        responses.GET, 'http://mock/shows/1390',
        body=read('fixtures/shows/1390.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    show = Trakt['shows'].get(1390)

    assert show.title == 'Game of Thrones'
    assert show.year == 2011

    assert show.pk == ('tvdb', '121361')
    assert show.keys == [
        ('tvdb', '121361'),
        ('tmdb', '1399'),
        ('imdb', 'tt0944947'),
        ('tvrage', '24493'),
        ('slug', 'game-of-thrones'),
        ('trakt', '1390')
    ]


@responses.activate
def test_show_not_found():
    responses.add(
        responses.GET, 'http://mock/shows/0',
        body='Not Found', status=404,
        content_type='text/html'
    )

    Trakt.base_url = 'http://mock'

    show = Trakt['shows'].get(0)
    assert show is None


@responses.activate
def test_seasons():
    responses.add(
        responses.GET, 'http://mock/shows/tt0944947/seasons',
        body=read('fixtures/shows/tt0944947/seasons.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    seasons = Trakt['shows'].seasons('tt0944947')

    assert len(seasons) == 6

    assert seasons[0].pk == 0
    assert seasons[0].keys == [
        0,
        ('tvdb', '137481'),
        ('tmdb', '3627'),
        ('trakt', '3962')
    ]

    assert seasons[5].pk == 5
    assert seasons[5].keys == [
        5,
        ('tmdb', '62090'),
        ('trakt', '3967')
    ]


@responses.activate
def test_seasons_extended():
    def callback(request):
        uri = urlparse(request.url)
        parameters = dict(parse_qsl(uri.query))

        if parameters.get('extended') != 'episodes':
            return 400, {}, None

        return 200, {}, read('fixtures/shows/tt0944947/seasons_extended.json')

    responses.add_callback(
        responses.GET, 'http://mock/shows/tt0944947/seasons',
        callback=callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    seasons = Trakt['shows'].seasons('tt0944947', extended='episodes')

    assert len(seasons) == 6

    # Specials
    assert seasons[0].pk == 0
    assert seasons[0].keys == [
        0,
        ('tvdb', '137481'),
        ('tmdb', '3627'),
        ('trakt', '3962')
    ]

    assert len(seasons[0].episodes) == 14

    assert seasons[0].episodes[5].pk == (0, 5)
    assert seasons[0].episodes[5].title == '2011 Comic Con Panel'

    assert seasons[0].episodes[13].pk == (0, 13)
    assert seasons[0].episodes[13].title == 'World Premiere'

    # Season 5
    assert seasons[5].pk == 5
    assert seasons[5].keys == [
        5,
        ('tmdb', '62090'),
        ('trakt', '3967')
    ]

    assert len(seasons[5].episodes) == 10

    assert seasons[5].episodes[7].pk == (5, 7)
    assert seasons[5].episodes[7].title == 'The Gift'

    assert seasons[5].episodes[9].pk == (5, 9)
    assert seasons[5].episodes[9].title == 'The Dance of Dragons'


@responses.activate
def test_season():
    responses.add(
        responses.GET, 'http://mock/shows/game-of-thrones/seasons/1',
        body=read('fixtures/shows/game-of-thrones/seasons/1.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    episodes = Trakt['shows'].season('game-of-thrones', 1)

    assert len(episodes) == 10

    assert episodes[0].title == 'Winter Is Coming'
    assert episodes[0].pk == (1, 1)
    assert episodes[0].keys == [
        (1, 1),
        ('tvdb', '3254641'),
        ('tmdb', '63056'),
        ('imdb', 'tt1480055'),
        ('tvrage', '1065008299'),
        ('trakt', '73640')
    ]

    assert episodes[9].title == 'Fire and Blood'
    assert episodes[9].pk == (1, 10)
    assert episodes[9].keys == [
        (1, 10),
        ('tvdb', '4063491'),
        ('tmdb', '63065'),
        ('imdb', 'tt1851397'),
        ('tvrage', '1065039307'),
        ('trakt', '73649')
    ]


@responses.activate
def test_season_not_found():
    responses.add(
        responses.GET, 'http://mock/shows/fake-show/seasons/1',
        body='Not Found', status=404,
        content_type='text/html'
    )

    Trakt.base_url = 'http://mock'

    season = Trakt['shows'].season('fake-show', 1)
    assert season is None


@responses.activate
def test_episode_basic():
    responses.add(
        responses.GET, 'http://mock/shows/game-of-thrones/seasons/1/episodes/1',
        body=read('fixtures/shows/game-of-thrones/seasons/1/episodes/1.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    episode = Trakt['shows'].episode('game-of-thrones', 1, 1)

    assert episode.title == 'Winter Is Coming'
    assert episode.pk == (1, 1)
    assert episode.keys == [
        (1, 1),
        ('tvdb', '3254641'),
        ('tmdb', '63056'),
        ('imdb', 'tt1480055'),
        ('tvrage', '1065008299'),
        ('trakt', '73640')
    ]


@responses.activate
def test_episode_not_found():
    responses.add(
        responses.GET, 'http://mock/shows/fake-show/seasons/1/episodes/1',
        body='{"error": "not found"}', status=404,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    episode = Trakt['shows'].episode('fake-show', 1, 1)
    assert episode is None


@responses.activate
def test_episode_proxy():
    responses.add(
        responses.GET, 'http://mock/shows/game-of-thrones/seasons/1/episodes/1',
        body=read('fixtures/shows/game-of-thrones/seasons/1/episodes/1.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    episode = Trakt['shows/game-of-thrones'].episode(1, 1)

    assert episode.title == 'Winter Is Coming'
    assert episode.pk == (1, 1)
    assert episode.keys == [
        (1, 1),
        ('tvdb', '3254641'),
        ('tmdb', '63056'),
        ('imdb', 'tt1480055'),
        ('tvrage', '1065008299'),
        ('trakt', '73640')
    ]



from trakt import Trakt
import json
import pytest
import responses


@responses.activate
def test_code():
    expected_request = {
        'client_id': 'mock-client_id'
    }

    def callback(request):
        data = json.loads(request.body)

        if data != expected_request:
            return 403, {}, ''

        return 200, {}, json.dumps({
            "device_code": "mock-device_code",
            "user_code": "mock-user_code",
            "verification_url": "https://trakt.tv/activate",
            "expires_in": 600,
            "interval": 5
        })

    responses.add_callback(
        responses.POST, 'http://api.mock/oauth/device/code',
        callback=callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://api.mock'

    # Validate `code` request/response
    with Trakt.configuration.client('mock-client_id', 'mock-client_secret'):
        assert Trakt['oauth/device'].code() == {
            "device_code": "mock-device_code",
            "user_code": "mock-user_code",
            "verification_url": "https://trakt.tv/activate",
            "expires_in": 600,
            "interval": 5
        }

    # Ensure `code` raises a `ValueError` on incorrect configuration
    with pytest.raises(ValueError):
        assert Trakt['oauth/device'].code()


@responses.activate
def test_token():
    expected_request = {
        'client_id': 'mock-client_id',
        'client_secret': 'mock-client_secret',

        'code': 'mock-device_code'
    }

    def callback(request):
        data = json.loads(request.body)

        if data != expected_request:
            return 403, {}, ''

        return 200, {}, json.dumps({
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        })

    responses.add_callback(
        responses.POST, 'http://api.mock/oauth/device/token',
        callback=callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://api.mock'

    # Validate `token` request/response
    with Trakt.configuration.client('mock-client_id', 'mock-client_secret'):
        assert Trakt['oauth/device'].token('mock-device_code') == {
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        }

    # Ensure `token` raises a `ValueError` on incorrect configuration
    with pytest.raises(ValueError):
        assert Trakt['oauth/device'].token('mock-device_code')



from trakt.objects.comment import Comment
from trakt.objects.episode import Episode
from trakt.objects.list import CustomList, List
from trakt.objects.media import Media
from trakt.objects.movie import Movie
from trakt.objects.rating import Rating
from trakt.objects.season import Season
from trakt.objects.show import Show
from trakt.objects.video import Video
from trakt.objects.person import Person


__all__ = [
    'Comment',
    'Episode',
    'CustomList', 'List',
    'Media',
    'Movie',
    'Rating',
    'Season',
    'Show',
    'Video',
    'Person'
]



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/sync/ratings/movies',
        callback=authenticated_response('fixtures/sync/ratings/movies.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        collection = Trakt['sync/ratings'].movies()

    # Ensure collection is valid
    assert_that(collection, not_none())

    # 100 Bloody Acres (2012)
    assert_that(collection[('imdb', 'tt2290065')], has_properties({
        'pk': ('imdb', 'tt2290065'),
        'title': '100 Bloody Acres',
        'year': 2012,

        'rating': has_properties({
            'value': 8,
            'timestamp': datetime(2015, 1, 28, 2, 26, 37, tzinfo=tzutc())
        }),

        # Keys
        'keys': [
            ('imdb', 'tt2290065'),
            ('tmdb', '126757'),
            ('slug', '100-bloody-acres-2012'),
            ('trakt', '86920')
        ]
    }))

    # The Hobbit: The Desolation of Smaug (2013)
    assert_that(collection[('imdb', 'tt1170358')], has_properties({
        'pk': ('imdb', 'tt1170358'),
        'title': 'The Hobbit: The Desolation of Smaug',
        'year': 2013,

        'rating': has_properties({
            'value': 10,
            'timestamp': datetime(2014, 11, 1, 0, 24, 54, tzinfo=tzutc())
        }),

        # Keys
        'keys': [
            ('imdb', 'tt1170358'),
            ('tmdb', '57158'),
            ('slug', 'the-hobbit-the-desolation-of-smaug-2013'),
            ('trakt', '40808')
        ]
    }))


@responses.activate
def test_rating_filter():
    responses.add_callback(
        responses.GET, 'http://mock/sync/ratings/movies/8',
        callback=authenticated_response('fixtures/sync/ratings/movies/8.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        collection = Trakt['sync/ratings'].movies(rating=8)

    # Ensure collection is valid
    assert_that(collection, not_none())
    assert_that(collection, has_length(1))

    # 100 Bloody Acres (2012)
    assert_that(collection[('imdb', 'tt2290065')], has_properties({
        'pk': ('imdb', 'tt2290065'),
        'title': '100 Bloody Acres',
        'year': 2012,

        'rating': has_properties({
            'value': 8,
            'timestamp': datetime(2015, 1, 28, 2, 26, 37, tzinfo=tzutc())
        }),

        # Keys
        'keys': [
            ('imdb', 'tt2290065'),
            ('tmdb', '126757'),
            ('slug', '100-bloody-acres-2012'),
            ('trakt', '86920')
        ]
    }))



from trakt.interfaces.base import Interface


class UsersSettingsInterface(Interface):
    path = 'users/settings'

    def get(self):
        response = self.http.get()

        return self.get_data(response)



from trakt.core.configuration import DEFAULT_HTTP_RETRY, DEFAULT_HTTP_MAX_RETRIES, DEFAULT_HTTP_TIMEOUT, \
    DEFAULT_HTTP_RETRY_SLEEP
from trakt.core.context_stack import ContextStack
from trakt.core.helpers import synchronized
from trakt.core.request import TraktRequest

from requests.adapters import HTTPAdapter, DEFAULT_POOLBLOCK
from threading import Lock
import calendar
import datetime
import logging
import requests
import socket
import time

try:
    import ssl
except ImportError:
    ssl = None

log = logging.getLogger(__name__)


class HttpClient(object):
    def __init__(self, client, adapter_kwargs=None):
        self.client = client
        self.adapter_kwargs = adapter_kwargs or {}

        # Build client
        self.configuration = ContextStack()
        self.session = None

        self._proxies = {}
        self._validate_oauth_lock = Lock()

        self.rebuild()

    @property
    def proxies(self):
        if self.session and self.session.proxies:
            return self.session.proxies

        return self._proxies

    @proxies.setter
    def proxies(self, proxies):
        if self.session:
            self.session.proxies = proxies

        self._proxies = proxies

    def configure(self, path=None):
        self.configuration.push(base_path=path)

        return self

    def request(self, method, path=None, params=None, data=None, query=None, authenticated=False, **kwargs):
        # Retrieve configuration
        ctx = self.configuration.pop()

        # Build request
        request = TraktRequest(
            self.client,
            method=method,

            path=self._build_path(ctx, path),
            params=params,

            data=data,
            query=query,

            authenticated=authenticated,
            **kwargs
        )

        # Validate authentication details (OAuth)
        if authenticated and not self.validate():
            return None

        # Prepare request
        prepared = request.prepare()

        # Send request
        return self.send(prepared)

    def send(self, request):
        # Retrieve http configuration
        retry = self.client.configuration.get('http.retry', DEFAULT_HTTP_RETRY)
        max_retries = self.client.configuration.get('http.max_retries', DEFAULT_HTTP_MAX_RETRIES)
        retry_sleep = self.client.configuration.get('http.retry_sleep', DEFAULT_HTTP_RETRY_SLEEP)
        timeout = self.client.configuration.get('http.timeout', DEFAULT_HTTP_TIMEOUT)

        # Send request
        response = None

        for i in range(max_retries + 1):
            if i > 0:
                log.warn('Retry # %s', i)

            # Send request
            try:
                response = self.session.send(request, timeout=timeout)
            except socket.gaierror as e:
                code, _ = e

                if code != 8:
                    raise e

                log.warn('Encountered socket.gaierror (code: 8)')

                response = self.rebuild().send(request, timeout=timeout)

            # Retry requests on errors >= 500 (when enabled)
            if not retry or response.status_code < 500:
                break

            log.warn('Continue retry since status is %s, waiting %s seconds', response.status_code, retry_sleep)
            time.sleep(retry_sleep)

        return response

    def delete(self, path=None, params=None, data=None, **kwargs):
        return self.request('DELETE', path, params, data, **kwargs)

    def get(self, path=None, params=None, data=None, **kwargs):
        return self.request('GET', path, params, data, **kwargs)

    def post(self, path=None, params=None, data=None, **kwargs):
        return self.request('POST', path, params, data, **kwargs)

    def put(self, path=None, params=None, data=None, **kwargs):
        return self.request('PUT', path, params, data, **kwargs)

    def rebuild(self):
        if self.session:
            log.info('Rebuilding session and connection pools...')

        # Build the connection pool
        self.session = requests.Session()
        self.session.proxies = self.proxies

        # Mount adapters
        self.session.mount('http://', HTTPAdapter(**self.adapter_kwargs))

        if ssl is not None:
            self.session.mount('https://', HTTPSAdapter(ssl_version=ssl.PROTOCOL_TLSv1, **self.adapter_kwargs))
        else:
            log.warn('"ssl" module is not available, unable to change "ssl_version"')
            self.session.mount('https://', HTTPSAdapter(**self.adapter_kwargs))

        return self.session

    def validate(self):
        config = self.client.configuration

        # xAuth
        if config['auth.login'] and config['auth.token']:
            return True

        # OAuth
        if config['oauth.token']:
            # Validate OAuth token, refresh if needed
            return self._validate_oauth()

        return False

    def _build_path(self, ctx, path):
        if not ctx:
            # No context available
            return path

        if ctx.base_path and path:
            # Prepend `base_path` to relative `path`s
            if not path.startswith('/'):
                path = ctx.base_path + '/' + path
        elif ctx.base_path:
            # Set path to `base_path
            path = ctx.base_path

        return path

    @synchronized(lambda self: self._validate_oauth_lock)
    def _validate_oauth(self):
        config = self.client.configuration

        if config['oauth.created_at'] is None or config['oauth.expires_in'] is None:
            log.debug('OAuth - Missing "created_at" or "expires_in" parameter, '
                      'unable to determine if token is still valid')
            return True

        current = calendar.timegm(datetime.datetime.utcnow().utctimetuple())
        expires_at = config['oauth.created_at'] + config['oauth.expires_in'] - (48 * 60 * 60)

        if current < expires_at:
            return True

        if not config['oauth.refresh']:
            log.warn('OAuth - Unable to refresh expired token (token refreshing hasn\'t been enabled)')
            return False

        if not config['oauth.refresh_token']:
            log.warn('OAuth - Unable to refresh expired token ("refresh_token" is parameter is missing)')
            return False

        # Refresh token
        response = self.client['oauth'].token_refresh(config['oauth.refresh_token'], 'urn:ietf:wg:oauth:2.0:oob')

        if not response:
            log.warn('OAuth - Unable to refresh expired token (error occurred while trying to refresh the token)')
            return False

        # Update current configuration
        config.current.oauth.from_response(response)

        # Fire refresh event
        self.client.emit('oauth.token_refreshed', response)
        return True


class HTTPSAdapter(HTTPAdapter):
    def __init__(self, ssl_version=None, *args, **kwargs):
        self._ssl_version = ssl_version

        super(HTTPSAdapter, self).__init__(*args, **kwargs)

    def init_poolmanager(self, connections, maxsize, block=DEFAULT_POOLBLOCK, **pool_kwargs):
        pool_kwargs['ssl_version'] = self._ssl_version

        return super(HTTPSAdapter, self).init_poolmanager(
            connections, maxsize, block,
            **pool_kwargs
        )



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from trakt import Trakt
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists',
        callback=authenticated_response('fixtures/users/me/lists.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        lists = Trakt['users/me/lists'].get()

        assert lists is not None

        # Resolve lists
        lists = list(lists)

    # Validate list container
    assert len(lists) == 1

    # Validate movies list
    movies_list = lists[0]

    assert movies_list.name == 'Movies'
    assert movies_list.description is None
    assert movies_list.likes == 0

    assert movies_list.allow_comments is True
    assert movies_list.display_numbers is False

    assert movies_list.updated_at == datetime(2015, 6, 22, 2, 25, tzinfo=tzutc())

    assert movies_list.comment_count == 0
    assert movies_list.item_count == 2

    assert movies_list.privacy == 'private'



from helpers import authenticate

import logging
import os


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)

    from trakt import Trakt

    # Configure
    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID'),
        secret=os.environ.get('CLIENT_SECRET')
    )

    # Authenticate
    Trakt.configuration.defaults.oauth.from_response(
        authenticate()
    )

    for x, liked_list in enumerate(Trakt['users'].likes('lists', pagination=True)):
        print '[%s] %r' % (x + 1, liked_list)

        items = liked_list.items()

        if not items:
            print ' - ERROR'
            continue

        items = list(items)
        print ' - %d item(s)' % len(items)



from trakt.core.helpers import to_iso8601, deprecated
from trakt.objects.core.helpers import update_attributes
from trakt.objects.media import Media

from six import iteritems


class Show(Media):
    def __init__(self, client, keys, index=None):
        super(Show, self).__init__(client, keys, index)

        self.title = None
        """
        :type: :class:`~python:str`

        Show title
        """

        self.year = None
        """
        :type: :class:`~python:int`

        Show year
        """

        self.seasons = {}
        """
        :type: :class:`~python:dict`

        Seasons, defined as :code:`{season_num: Season}`

        **Note:** this field might not be available with some methods
        """

        self.watchers = None
        """
        :type: :class:`~python:int`

        Number of active watchers (returned by the :code:`Trakt['movies'].trending()`
        and :code:`Trakt['shows'].trending()` methods)
        """

    def episodes(self):
        """Returns a flat episode iterator

        :returns: Iterator :code:`((season_num, episode_num), Episode)`
        :rtype: iterator
        """

        for sk, season in iteritems(self.seasons):
            # Yield each episode in season
            for ek, episode in iteritems(season.episodes):
                yield (sk, ek), episode

    def to_identifier(self):
        """Returns the show identifier which is compatible with requests that require
        show definitions.

        :return: Show identifier/definition
        :rtype: :class:`~python:dict`
        """

        return {
            'ids': dict(self.keys),
            'title': self.title,
            'year': self.year
        }

    @deprecated('Show.to_info() has been moved to Show.to_dict()')
    def to_info(self):
        """**Deprecated:** use the :code:`to_dict()` method instead"""
        return self.to_dict()

    def to_dict(self):
        """Dump show to a dictionary

        :return: Show dictionary
        :rtype: :class:`~python:dict`
        """

        result = self.to_identifier()

        result['seasons'] = [
            season.to_dict()
            for season in self.seasons.values()
        ]

        if self.rating:
            result['rating'] = self.rating.value
            result['rated_at'] = to_iso8601(self.rating.timestamp)

        result['in_watchlist'] = self.in_watchlist if self.in_watchlist is not None else 0

        return result

    def _update(self, info=None, **kwargs):
        super(Show, self)._update(info, **kwargs)

        update_attributes(self, info, [
            'title',

            'watchers'  # trending
        ])

        if info.get('year'):
            self.year = int(info['year'])

    @classmethod
    def _construct(cls, client, keys, info=None, index=None, **kwargs):
        show = cls(client, keys, index=index)
        show._update(info, **kwargs)

        return show

    def __repr__(self):
        return '<Show %r (%s)>' % (self.title, self.year)



from hamcrest import *
from trakt import Trakt
import responses


@responses.activate
def test_missing_media_parameter():
    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        assert_that(calling(Trakt['sync/watched'].get), raises(ValueError))



from trakt.core import exceptions
from trakt.interfaces.base import Interface

import pytest
import responses
import requests


@responses.activate
def test_content_type():
    responses.add(
        responses.GET, 'http://mock/test',
        body='{"test": True}', status=502,
        content_type=None
    )

    r = requests.get('http://mock/test')

    data = Interface(None).get_data(r)
    assert data is None


@responses.activate
def test_server_error():
    responses.add(
        responses.GET, 'http://mock/test',
        body='{"test": True}', status=502,
        content_type=None
    )

    r = requests.get('http://mock/test')

    with pytest.raises(exceptions.ServerError):
        Interface(None).get_data(r, exceptions=True)


@responses.activate
def test_client_error():
    responses.add(
        responses.GET, 'http://mock/test',
        body='{"test": True}', status=404,
        content_type=None
    )

    r = requests.get('http://mock/test')

    with pytest.raises(exceptions.ClientError):
        Interface(None).get_data(r, exceptions=True)



from helpers import authenticate

from trakt import Trakt
from trakt.objects import Movie, Show
import logging
import os

logging.basicConfig(level=logging.DEBUG)


if __name__ == '__main__':
    # Configure
    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID'),
        secret=os.environ.get('CLIENT_SECRET')
    )

    Trakt.configuration.defaults.http(
        retry=True
    )

    # Authenticate
    Trakt.configuration.defaults.oauth.from_response(
        authenticate()
    )

    # Fetch playback for movies
    playback = Trakt['sync/playback'].movies(exceptions=True)

    for key, item in playback.items():
        print item

        if type(item) is Movie:
            print '\tprogress: %r' % item.progress
            print '\tpaused_at: %r' % item.paused_at

    # Fetch movie library (watched, collection, ratings)
    movies = {}

    Trakt['sync/watched'].movies(movies, exceptions=True)
    Trakt['sync/collection'].movies(movies, exceptions=True)

    Trakt['sync/ratings'].movies(movies, exceptions=True)

    for key, movie in movies.items():
        print movie

        print '\t', 'keys', '\t' * 3, movie.keys
        print '\t', 'rating', '\t' * 3, movie.rating

        print '\t', 'is_watched', '\t' * 2, movie.is_watched
        print '\t', 'last_watched_at', '\t', movie.last_watched_at
        print '\t', 'is_collected', '\t', movie.is_collected
        print '\t', 'collected_at', '\t', movie.collected_at
        print '\t', 'plays', '\t' * 3, movie.plays

        print

    # Fetch playback for shows
    playback = Trakt['sync/playback'].episodes(exceptions=True)

    for key, item in playback.items():
        print item

        if type(item) is Show:
            for (sk, ek), episode in item.episodes():
                print '\t', episode

                print '\t\tprogress: %r' % episode.progress
                print '\t\tpaused_at: %r' % episode.paused_at

    # Fetch show/episode library (watched, collection, ratings)
    shows = {}

    Trakt['sync/watched'].shows(shows, exceptions=True)
    Trakt['sync/collection'].shows(shows, exceptions=True)

    Trakt['sync/ratings'].shows(shows, exceptions=True)
    Trakt['sync/ratings'].episodes(shows, exceptions=True)

    for key, show in shows.items():
        print show

        print '\t', 'keys', '\t' * 3, show.keys
        print '\t', 'rating', '\t' * 3, show.rating
        print



from tests.core.helpers import read

from trakt import Trakt
import responses


@responses.activate
def test_movie():
    responses.add(
        responses.GET, 'http://mock/movies/trending',
        body=read('fixtures/movies/trending.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    movies = Trakt['movies'].trending()

    assert len(movies) == 10

    assert [(m.watchers, m.title, m.year) for m in movies] == [
        (24, "Cinderella", 2015),
        (15, "Pleasure or Pain", 2013),
        (14, "Get Hard", 2015),
        (12, "Mad Max: Fury Road", 2015),
        (11, "Maggie", 2015),
        (10, "The Gunman", 2015),
        (9, "Ex Machina", 2015),
        (8, "The SpongeBob Movie: Sponge Out of Water", 2015),
        (7, "Chappie", 2015),
        (6, "While We're Young", 2015)
    ]


@responses.activate
def test_show():
    responses.add(
        responses.GET, 'http://mock/shows/trending',
        body=read('fixtures/shows/trending.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    shows = Trakt['shows'].trending()

    assert len(shows) == 10

    assert [(m.watchers, m.title, m.year) for m in shows] == [
        (45, "Orange Is the New Black", 2013),
        (42, "Game of Thrones", 2011),
        (35, "True Detective", 2014),
        (21, "Pretty Little Liars", 2010),
        (21, "Sense8", 2015),
        (21, "The Last Ship", 2014),
        (17, "Arrow", 2012),
        (14, "The Big Bang Theory", 2007),
        (13, "Tyrant", 2014),
        (12, "Grey's Anatomy", 2005)
    ]



from threading import Event
from trakt import Trakt
import json
import pytest
import responses


def test_expired():
    expired = Event()

    def on_expired():
        expired.set()

    # Setup client
    Trakt.base_url = 'http://api.mock'

    # Construct poller
    poller = Trakt['oauth/device'].poll('mock-device_code', -5, 5)
    poller.on('expired', on_expired)

    # Start poller
    poller.start()

    # Ensure "expired" event was fired
    expired.wait(3)

    assert expired.is_set() is True


def test_aborted():
    aborted = Event()

    def on_aborted():
        aborted.set()

    # Setup client
    Trakt.base_url = 'http://api.mock'

    # Construct poller
    poller = Trakt['oauth/device'].poll('mock-device_code', 600, 5)
    poller.on('aborted', on_aborted)

    # Start poller
    poller.start()

    # Ensure "aborted" event was fired
    aborted.wait(3)

    assert aborted.is_set() is True


@responses.activate
def test_poll_authenticated():
    class state:
        count = 0

    def callback(request):
        if state.count < 5:
            return 403, {}, ''

        return 200, {}, json.dumps({
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        })

    responses.add_callback(
        responses.POST, 'http://api.mock/oauth/device/token',
        callback=callback,
        content_type='application/json'
    )

    authenticated = Event()

    def on_authenticated(token):
        if not token:
            return

        authenticated.set()

    def on_poll(cb):
        state.count += 1

        cb()

    # Setup client
    Trakt.base_url = 'http://api.mock'

    # Construct poller
    poller = Trakt['oauth/device'].poll('mock-device_code', 600, 1)
    poller.on('authenticated', on_authenticated)
    poller.on('poll', on_poll)

    # Set client configuration
    Trakt.configuration.defaults.client('mock-client_id', 'mock-client_secret')

    try:
        # Start poller
        poller.start()

        # Ensure "authenticated" event was fired
        authenticated.wait(10)

        assert authenticated.is_set() is True
    finally:
        # Reset client configuration
        Trakt.configuration.defaults.client()


@responses.activate
def test_poll_expired():
    class state:
        count = 0

    def callback(request):
        if state.count < 5:
            return 403, {}, ''

        return 200, {}, json.dumps({
            "access_token": "mock-access_token",
            "token_type": "bearer",
            "expires_in": 7200,
            "refresh_token": "mock-refresh_token",
            "scope": "public"
        })

    responses.add_callback(
        responses.POST, 'http://api.mock/oauth/device/token',
        callback=callback,
        content_type='application/json'
    )

    expired = Event()

    def on_expired():
        expired.set()

    def on_poll(cb):
        state.count += 1

        cb()

    # Setup client
    Trakt.base_url = 'http://api.mock'

    # Construct poller
    poller = Trakt['oauth/device'].poll('mock-device_code', 2, 1)
    poller.on('expired', on_expired)
    poller.on('poll', on_poll)

    # Set client configuration
    Trakt.configuration.defaults.client('mock-client_id', 'mock-client_secret')

    try:
        # Start poller
        poller.start()

        # Ensure "authenticated" event was fired
        expired.wait(10)

        assert expired.is_set() is True
    finally:
        # Reset client configuration
        Trakt.configuration.defaults.client()


def test_double_start():
    # Setup client
    Trakt.base_url = 'http://api.mock'

    # Construct poller
    poller = Trakt['oauth/device'].poll('mock-device_code', 600, 5)

    # Start poller
    poller.start()

    try:
        # Ensure an exception is raised when attempting to start the poller again
        with pytest.raises(Exception):
            poller.start()
    finally:
        # Stop polling
        poller.stop()



import logging
import os

logging.basicConfig(level=logging.DEBUG)

from trakt import Trakt
from trakt.objects import Season, Episode


def print_lookup(id, service):
    print "Trakt['search'].lookup(%r, %r)" % (id, service)

    item = Trakt['search'].lookup(id, service)

    if type(item) is Episode and item.show:
        sk, ek = item.pk
        print "\t%s (%s) - S%02dE%02d %r" % (item.show.title, item.show.year, sk, ek, item.title)
    else:
        print "\t%s (%s)" % (item.title, item.year)


def print_query(query, media=None, year=None):
    print "Trakt['search'].query(%r, %r, %r)" % (query, media, year)

    items = Trakt['search'].query(query, media, year)

    for item in items:
        if type(item) is Episode and item.show:
            sk, ek = item.pk
            print "\t[%.2d%%] %s (%s) - S%02dE%02d %r" % (item.score, item.show.title, item.show.year, sk, ek, item.title)
        else:
            print "\t[%.2d%%] %s (%s)" % (item.score, item.title, item.year)


if __name__ == '__main__':
    # Configure
    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID')
    )

    # Lookup by id
    print_lookup('tt0848228', 'imdb')
    print_lookup('tt0903747', 'imdb')
    print_lookup('tt0959621', 'imdb')

    # Search by name
    print_query('The Avengers', 'movie')
    print_query('Breaking Bad', 'show')
    print_query('Breaking Bad', 'episode')



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
import responses


@responses.activate
def test_ratings():
    responses.add_callback(
        responses.GET, 'http://mock/sync/ratings/shows',
        callback=authenticated_response('fixtures/sync/ratings/shows.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.GET, 'http://mock/sync/ratings/seasons',
        callback=authenticated_response('fixtures/sync/ratings/seasons.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.GET, 'http://mock/sync/ratings/episodes',
        callback=authenticated_response('fixtures/sync/ratings/episodes.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    collection = {}

    with Trakt.configuration.auth('mock', 'mock'):
        Trakt['sync/ratings'].shows(collection)
        Trakt['sync/ratings'].seasons(collection)
        Trakt['sync/ratings'].episodes(collection)

    # Ensure collection is valid
    assert_that(collection, not_none())
    assert_that(collection, has_length(6))

    # Chuck (2007)
    assert_that(collection[('tvdb', '80348')], has_properties({
        'pk': ('tvdb', '80348'),
        'title': 'Chuck',
        'year': 2007,

        'rating': has_properties({
            'value': 10,
            'timestamp': datetime(2014, 10, 19, 23, 2, 23, tzinfo=tzutc())
        }),

        # Seasons
        'seasons': all_of(
            has_length(1),
            has_entry(1, has_properties({
                'pk': 1,

                # Rating
                'rating': has_properties({
                    'value': 10,
                    'timestamp': datetime(2015, 3, 11, 23, 29, 35, tzinfo=tzutc())
                }),

                # Episodes
                'episodes': all_of(
                    has_length(1),
                    has_entry(1, has_properties({
                        'pk': (1, 1),

                        # Rating
                        'rating': has_properties({
                            'value': 10,
                            'timestamp': datetime(2014, 10, 19, 23, 2, 24, tzinfo=tzutc())
                        }),

                        # Keys
                        'keys': [
                            (1, 1),
                            ('tvdb', '332179'),
                            ('tmdb', '63425'),
                            ('tvrage', '579282'),
                            ('trakt', '74041')
                        ]
                    }))
                ),

                # Keys
                'keys': [
                    1,
                    ('tvdb', '27985'),
                    ('tmdb', '3650'),
                    ('trakt', '3993')
                ]
            })),
        ),

        # Keys
        'keys': [
            ('tvdb', '80348'),
            ('tmdb', '1404'),
            ('imdb', 'tt0934814'),
            ('tvrage', '15614'),
            ('slug', 'chuck'),
            ('trakt', '1395')
        ]
    }))



from threading import Condition, Lock


class Semaphore(object):
    def __init__(self, value=1):
        if value < 0:
            raise ValueError("semaphore initial value must be >= 0")

        self.__cond = Condition(Lock())
        self.__value = value

    def acquire(self, blocking=1):
        rc = False

        with self.__cond:
            while self.__value == 0:
                if not blocking:
                    break

                self.__cond.wait()
            else:
                self.__value = self.__value - 1

                rc = True

        return rc

    __enter__ = acquire

    def release(self):
        with self.__cond:
            self.__value = self.__value + 1
            self.__cond.notify()

    def release_multi(self, value):
        with self.__cond:
            self.__value = value
            self.__cond.notify_all()

    def __exit__(self, t, v, tb):
        self.release()



from trakt.interfaces.base import Interface
from trakt.mapper import CommentMapper, ListMapper

# Import child interfaces
from trakt.interfaces.users.lists import UsersListInterface, UsersListsInterface
from trakt.interfaces.users.settings import UsersSettingsInterface

import logging

log = logging.getLogger(__name__)

__all__ = [
    'UsersInterface',
    'UsersListsInterface',
    'UsersListInterface',
    'UsersSettingsInterface'
]


class UsersInterface(Interface):
    path = 'users'

    def likes(self, type=None, **kwargs):
        if type and type not in ['comments', 'lists']:
            raise ValueError('Unknown type specified: %r' % type)

        # Send request
        response = self.http.get('likes', params=[
            type
        ])

        if response.status_code < 200 or response.status_code >= 300:
            return

        # Parse response
        items = self.get_data(response, **kwargs)

        # Map items to comment/list objects
        for item in items:
            item_type = item.get('type')

            if item_type == 'comment':
                yield CommentMapper.comment(
                    self.client, item
                )
            elif item_type == 'list':
                yield ListMapper.custom_list(
                    self.client, item
                )
            else:
                log.warn('Unknown item returned, type: %r', item_type)



ERRORS = {
    400: ("Bad Request",            "Request couldn't be parsed"),
    401: ("Unauthorized",           "OAuth must be provided"),
    403: ("Forbidden",              "Invalid API key or unapproved app"),
    404: ("Not Found",              "Method exists, but no record found"),
    405: ("Method Not Found",       "Method doesn't exist"),
    409: ("Conflict",               "Resource already created"),
    412: ("Precondition Failed",    "Use application/json content type"),
    422: ("Unprocessible Entity",   "Validation error"),
    429: ("Rate Limit Exceeded",    "Rate limit exceeded"),

    500: ("Server Error",           "Server error"),
    502: ("Bad Gateway",            "Server unavailable"),
    503: ("Service Unavailable",    "Server overloaded (try again in 30s)"),
    504: ("Service Unavailable",    "Server overloaded (try again in 30s)"),

    520: ("Service Unavailable",    "CloudFlare: Web server is returning an unknown error"),
    521: ("Service Unavailable",    "CloudFlare: Web server is down"),
    522: ("Service Unavailable",    "CloudFlare: Connection timed out"),
    524: ("Service Unavailable",    "CloudFlare: A timeout occurred")
}



import logging
import os

logging.basicConfig(level=logging.DEBUG)

from trakt import Trakt

if __name__ == '__main__':
    # Configure
    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID')
    )

    # Login
    username = os.environ.get('USERNAME')
    token = os.environ.get('AUTH_TOKEN')

    if token is None:
        # Attempt authentication (retrieve new token)
        token = Trakt['auth'].login(username, os.environ.get('PASSWORD'))

    print 'Using token: %r' % token

    with Trakt.configuration.auth(username, token):
        print Trakt['sync/collection'].movies()

        with Trakt.configuration.http(retry=True):
            print Trakt['movies'].get('tron-legacy-2010')  # use only traktId, trakt slug or imdbId

            print Trakt['shows'].get(1390)  # use only traktId, trakt slug or imdbId

            print Trakt['shows'].seasons('tt0944947')
            print Trakt['shows'].season('game-of-thrones', 1)

            print Trakt['shows'].episode('game-of-thrones', 1, 1)



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
from trakt.objects import Person
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/people/items',
        callback=authenticated_response('fixtures/users/me/lists/people/items.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        items = Trakt['users/me/lists/people'].items()

    # Ensure collection is valid
    assert_that(items, not_none())

    # Validate items
    assert_that(items, contains(
        # Bryan Cranston
        all_of(
            instance_of(Person),
            has_properties({
                'pk': ('tmdb', '17419'),
                'name': 'Bryan Cranston',

                # Timestamps
                'listed_at': datetime(2014, 6, 17, 6, 52, 3, tzinfo=tzutc()),

                # Keys
                'keys': [
                    ('tmdb', '17419'),
                    ('imdb', 'nm0186505'),
                    ('tvrage', '1797'),
                    ('slug', 'bryan-cranston'),
                    ('trakt', '1')
                ]
            })
        ),

        # Aaron Paul
        all_of(
            instance_of(Person),
            has_properties({
                'pk': ('tmdb', '84497'),
                'name': 'Aaron Paul',

                # Timestamps
                'listed_at': datetime(2014, 6, 17, 6, 52, 3, tzinfo=tzutc()),

                # Keys
                'keys': [
                    ('tmdb', '84497'),
                    ('imdb', 'nm0666739'),
                    ('tvrage', '1823'),
                    ('slug', 'aaron-paul'),
                    ('trakt', '415249')
                ]
            })
        )
    ))



import logging
logging.basicConfig(level=logging.DEBUG)

from trakt import Trakt
import os


class Application(object):
    def __init__(self):
        self.authorization = None

        # Bind trakt events
        Trakt.on('oauth.token_refreshed', self.on_token_refreshed)

    def run(self):
        self.authenticate()

        if not self.authorization:
            print 'Authentication required'
            exit(1)

        # Simulate expired token
        self.authorization['expires_in'] = 0

        # Test authenticated calls
        with Trakt.configuration.oauth.from_response(self.authorization):
            # Expired token, requests will return `None`
            print Trakt['sync/collection'].movies()

        with Trakt.configuration.oauth.from_response(self.authorization, refresh=True):
            # Expired token will be refreshed automatically (as `refresh=True`)
            print Trakt['sync/collection'].movies()

        with Trakt.configuration.oauth.from_response(self.authorization):
            # Current token is still valid
            print Trakt['sync/collection'].movies()

    def authenticate(self):
        # Request authentication
        print 'Navigate to %s' % Trakt['oauth/pin'].url()
        pin = raw_input('Pin: ')

        # Exchange `code` for `access_token`
        self.authorization = Trakt['oauth'].token_exchange(pin, 'urn:ietf:wg:oauth:2.0:oob')

        if not self.authorization:
            return False

        print 'Token exchanged - authorization: %r' % self.authorization
        return True

    def on_token_refreshed(self, response):
        # OAuth token refreshed, save token for future calls
        self.authorization = response

        print 'Token refreshed - authorization: %r' % self.authorization

if __name__ == '__main__':
    # Configure
    Trakt.base_url = 'http://api.staging.trakt.tv'

    Trakt.configuration.defaults.app(
        id=os.environ.get('APP_ID')
    )

    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID'),
        secret=os.environ.get('CLIENT_SECRET')
    )

    # Start application
    app = Application()
    app.run()



from trakt.core.configuration import ConfigurationManager
from tests.core.semaphore import Semaphore

from threading import Thread
import threading
import time
import uuid


def test_threaded():
    id = uuid.uuid4()
    lock = Semaphore(value=0)

    configuration = ConfigurationManager()
    results = {}

    def test(key):
        with configuration.app(name='%s-test' % key):
            # Wait for signal
            lock.acquire()

            with configuration.app(version='%s-1.0' % key):
                results[threading.current_thread().name] = {
                    'app.name': configuration.get('app.name'),
                    'app.version': configuration.get('app.version')
                }

    one = Thread(target=test, name='%s.one' % id, args=(1, ))
    two = Thread(target=test, name='%s.two' % id, args=(2, ))

    one.start()
    two.start()

    # Wait for threads to properly startup
    time.sleep(1)

    # Wake up threads
    lock.release_multi(2)

    # Wait for threads to finish
    one.join()
    two.join()

    # Test results
    assert results['%s.one' % id] == {
        'app.name': '1-test',
        'app.version': '1-1.0',
    }

    assert results['%s.two' % id] == {
        'app.name': '2-test',
        'app.version': '2-1.0',
    }

    assert len(configuration.stack._threads) == 0



from trakt.interfaces.base import Interface
from trakt.mapper.summary import SummaryMapper


class ShowsInterface(Interface):
    path = 'shows'

    def get(self, id, **kwargs):
        response = self.http.get(
            str(id)
        )

        return SummaryMapper.show(
            self.client,
            self.get_data(response, **kwargs)
        )

    def trending(self, **kwargs):
        response = self.http.get(
            'trending'
        )

        return SummaryMapper.shows(
            self.client,
            self.get_data(response, **kwargs)
        )

    def next_episode(self, id, **kwargs):
        response = self.http.get(
            str(id), 'next_episode'
        )

        return SummaryMapper.episode(
            self.client,
            self.get_data(response, **kwargs)
        )

    def last_episode(self, id, **kwargs):
        response = self.http.get(
            str(id), 'last_episode'
        )

        return SummaryMapper.episode(
            self.client,
            self.get_data(response, **kwargs)
        )

    def seasons(self, id, extended=None, **kwargs):
        response = self.http.get(str(id), [
            'seasons'
        ], query={
            'extended': extended
        })

        return SummaryMapper.seasons(
            self.client,
            self.get_data(response, **kwargs)
        )

    def season(self, id, season, **kwargs):
        response = self.http.get(str(id), [
            'seasons', str(season)
        ])

        return SummaryMapper.episodes(
            self.client,
            self.get_data(response, **kwargs)
        )

    def episode(self, id, season, episode, **kwargs):
        response = self.http.get(str(id), [
            'seasons', str(season),
            'episodes', str(episode)
        ])

        return SummaryMapper.episode(
            self.client,
            self.get_data(response, **kwargs)
        )



from trakt.interfaces.base import Interface
from trakt.mapper.summary import SummaryMapper


class MoviesInterface(Interface):
    path = 'movies'

    def get(self, id, **kwargs):
        response = self.http.get(
            str(id)
        )

        # Parse response
        return SummaryMapper.movie(
            self.client,
            self.get_data(response, **kwargs)
        )

    def trending(self, **kwargs):
        response = self.http.get(
            'trending'
        )

        return SummaryMapper.movies(
            self.client,
            self.get_data(response, **kwargs)
        )



from tests.core.helpers import read

from six.moves.urllib.parse import parse_qsl, urlparse
from trakt import Trakt
from trakt.core.helpers import try_convert
import responses


@responses.activate
def test_iterator():
    def on_request(request):
        url = urlparse(request.url)
        parameters = dict(parse_qsl(url.query))

        page = try_convert(parameters.get('page'), int) or 1
        limit = try_convert(parameters.get('limit'), int)

        if limit is not None and limit != 2:
            # Invalid limit provided
            return 400, {}, ''

        return 200, {
            'X-Pagination-Limit':       '2',
            'X-Pagination-Item-Count':  '6',
            'X-Pagination-Page-Count':  '3'
        }, read('fixtures/users/me/lists_p%d.json' % page)

    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists',
        callback=on_request,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        lists = Trakt['users/me/lists'].get(pagination=True)

    # Resolve all pages
    items = list(lists)

    # Ensure items were returned correctly
    assert len(items) == 6

    assert [i.name for i in items] == [
        'Movies (1)',
        'Shows (1)',

        'Movies (2)',
        'Shows (2)',

        'Movies (3)',
        'Shows (3)'
    ]


@responses.activate
def test_invalid_content_type():
    def on_request(request):
        url = urlparse(request.url)
        parameters = dict(parse_qsl(url.query))

        page = try_convert(parameters.get('page'), int) or 1
        limit = try_convert(parameters.get('limit'), int)

        if limit is not None and limit != 2:
            # Invalid limit provided
            return 400, {}, ''

        return 200, {
            'X-Pagination-Limit':       '2',
            'X-Pagination-Item-Count':  '6',
            'X-Pagination-Page-Count':  '3'
        }, read('fixtures/users/me/lists_p%d.json' % page)

    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists',
        callback=on_request,
        content_type='text/plain'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        lists = Trakt['users/me/lists'].get(pagination=True)

    # Resolve all pages
    items = list(lists)

    # Ensure items were returned correctly
    assert len(items) == 0


@responses.activate
def test_invalid_json():
    def on_request(request):
        url = urlparse(request.url)
        parameters = dict(parse_qsl(url.query))

        page = try_convert(parameters.get('page'), int) or 1
        limit = try_convert(parameters.get('limit'), int)

        if limit is not None and limit != 2:
            # Invalid limit provided
            return 400, {}, ''

        if page == 2:
            return 200, {}, '<invalid-json-response>'

        return 200, {
            'X-Pagination-Limit':       '2',
            'X-Pagination-Item-Count':  '6',
            'X-Pagination-Page-Count':  '3'
        }, read('fixtures/users/me/lists_p%d.json' % page)

    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists',
        callback=on_request,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        lists = Trakt['users/me/lists'].get(pagination=True)

    # Resolve all pages
    items = list(lists)

    # Ensure items were returned correctly
    assert len(items) == 2


@responses.activate
def test_request_failure():
    def on_request(request):
        url = urlparse(request.url)
        parameters = dict(parse_qsl(url.query))

        page = try_convert(parameters.get('page'), int) or 1
        limit = try_convert(parameters.get('limit'), int)

        if limit is not None and limit != 2:
            # Invalid limit provided
            return 400, {}, ''

        if page == 2:
            return 400, {}, ''

        return 200, {
            'X-Pagination-Limit':       '2',
            'X-Pagination-Item-Count':  '6',
            'X-Pagination-Page-Count':  '3'
        }, read('fixtures/users/me/lists_p%d.json' % page)

    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists',
        callback=on_request,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        lists = Trakt['users/me/lists'].get(pagination=True)

    # Resolve all pages
    items = list(lists)

    # Ensure items were returned correctly
    assert len(items) == 2



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/sync/playback/movies',
        callback=authenticated_response('fixtures/sync/playback/movies.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        collection = Trakt['sync/playback'].movies()

    # Ensure collection is valid
    assert_that(collection, not_none())

    # TRON: Legacy (2010)
    assert_that(collection[('imdb', 'tt1104001')], has_properties({
        'pk': ('imdb', 'tt1104001'),
        'title': 'TRON: Legacy',
        'year': 2010,

        'progress': 64.0,

        # Timestamps
        'paused_at': datetime(2015, 2, 9, 5, 56, 58, tzinfo=tzutc()),

        # Keys
        'keys': [
            ('imdb',  'tt1104001'),
            ('tmdb',  '20526'),
            ('slug',  'tron-legacy-2010'),
            ('trakt', '12601')
        ]
    }))

    # 100 Bloody Acres (2012)
    assert_that(collection[('imdb', 'tt2290065')], has_properties({
        'pk': ('imdb', 'tt2290065'),
        'title': '100 Bloody Acres',
        'year': 2012,

        'progress': 0.0,

        # Timestamps
        'paused_at': datetime(2015, 1, 10, 6, 44, 9, tzinfo=tzutc()),

        # Keys
        'keys': [
            ('imdb', 'tt2290065'),
            ('tmdb', '126757'),
            ('slug', '100-bloody-acres-2012'),
            ('trakt', '86920')
        ]
    }))



from six.moves.urllib_parse import urlparse, parse_qsl
import json
import math
import os
import pytest
import six

if six.PY2:
    try:
        from six import cStringIO as BufferIO
    except ImportError:
        from six import StringIO as BufferIO
else:
    from io import BytesIO as BufferIO

TESTS_PATH = os.path.abspath(os.path.dirname(__file__) + os.path.sep + '..')


def read(path, mode='rb'):
    if not os.path.isabs(path):
        path = os.path.join(TESTS_PATH, path)

    path = os.path.abspath(path)

    with open(path, mode) as fp:
        return BufferIO(fp.read())


def authenticated_response(path=None, data=None):
    def callback(request):
        h = request.headers

        if h.get('trakt-user-login') == 'mock' and h.get('trakt-user-token') == 'mock':
            return 200, {}, read(path) if path else data

        if h.get('Authorization') == 'Bearer mock':
            return 200, {}, read(path) if path else data

        return 401, {}, ''

    return callback


def pagination_response(path=None, authenticated=False):
    if not os.path.isabs(path):
        path = os.path.join(TESTS_PATH, path)

    path = os.path.abspath(path)

    # Parse file
    with open(path, 'r') as fp:
        collection = json.load(fp)

    # Construct request callback
    def callback(request):
        if authenticated and not is_authenticated(request.headers):
            return 401, {}, ''

        # Parse url
        url = urlparse(request.url)
        query = dict(parse_qsl(url.query))

        # Retrieve parameters
        page = int(query.get('page', 1))
        limit = int(query.get('limit', 10))

        # Retrieve page items
        start = (page - 1) * limit
        end = page * limit

        items = collection[start:end]

        if not items:
            return 404, {}, ''

        # Return page response
        page_count = int(math.ceil(
            float(len(collection)) / limit
        ))

        return (
            200, {
                'Content-Type':             'application/json',
                'X-Pagination-Page':        str(page),
                'X-Pagination-Limit':       str(limit),
                'X-Pagination-Page-Count':  str(page_count),
                'X-Pagination-Item-Count':  str(len(collection))
            },
            json.dumps(items)
        )

    return callback


def is_authenticated(headers):
    if headers.get('trakt-user-login') == 'mock' and headers.get('trakt-user-token') == 'mock':
        return True

    if headers.get('Authorization') == 'Bearer mock':
        return True

    return False


def assert_url(url, expected_path, expected_query=None):
    __tracebackhide__ = True

    parsed = urlparse(url)
    query = dict(parse_qsl(parsed.query))

    if parsed.path != expected_path:
        pytest.fail("url.path is %r, expected %r" % (parsed.path, expected_path))

    if expected_query is not None and query != expected_query:
        pytest.fail('url.query is %r, expected %r' % (query, expected_query))



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
from trakt.objects import Show, Season, Episode
import responses


@responses.activate
def test_watched():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/shows/items',
        callback=authenticated_response('fixtures/users/me/lists/shows/items.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        items = Trakt['users/me/lists/shows'].items()

    # Ensure collection is valid
    assert_that(items, not_none())
    assert_that(items, has_length(3))

    # Validate items
    assert_that(items, contains(
        # Game of Thrones (2011)
        all_of(
            instance_of(Show),
            has_properties({
                'pk': ('tvdb', '121361'),
                'title': 'Game of Thrones',
                'year': 2011,

                # Keys
                'keys': [
                    ('tvdb', '121361'),
                    ('tmdb', '1399'),
                    ('imdb', 'tt0944947'),
                    ('tvrage', '24493'),
                    ('slug', 'game-of-thrones'),
                    ('trakt', '1390')
                ]
            })
        ),

        # Game of Thrones (2011) - S05
        all_of(
            instance_of(Season),
            has_properties({
                'pk': 5,

                # Keys
                'keys': [
                    5,
                    ('tmdb', '62090'),
                    ('trakt', '3967')
                ]
            })
        ),

        # Game of Thrones (2011) - S05E04
        all_of(
            instance_of(Episode),
            has_properties({
                'pk': (5, 4),

                # Keys
                'keys': [
                    (5, 4),
                    ('tvdb', '5150183'),
                    ('tmdb', '1045553'),
                    ('imdb', 'tt3866838'),
                    ('tvrage', '1065765456'),
                    ('trakt', '1782362')
                ]
            })
        )
    ))



from helpers import authenticate

from pprint import pprint
from trakt import Trakt
import logging
import os
import time

logging.basicConfig(level=logging.DEBUG)


if __name__ == '__main__':
    # Configure
    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID'),
        secret=os.environ.get('CLIENT_SECRET')
    )

    # Authenticate
    Trakt.configuration.defaults.oauth.from_response(
        authenticate()
    )

    # movie = {
    #     'title': 'Guardians of the Galaxy',
    #     'year': 2014,
    #     'ids': {
    #         'trakt': 28,
    #         'slug': 'guardians-of-the-galaxy-2014',
    #         'imdb': 'tt2015381',
    #         'tmdb': 118340
    #     }
    # }

    show = {
        "title": "Breaking Bad",
        "year": 2008,
        "ids": {
            "trakt": 1,
            "slug": "breaking-bad",
            "tvdb": 81189,
            "imdb": "tt0903747",
            "tmdb": 1396,
            "tvrage": 18164
        }
    }

    episode = {
        "season": 1,
        "number": 1,
        "title": "Pilot",
        "ids": {
            "trakt": 16,
            "tvdb": 349232,
            "imdb": "tt0959621",
            "tmdb": 62085,
            "tvrage": 637041
        }
    }

    # - Start watching
    pprint(Trakt['scrobble'].start(
        show=show,
        episode=episode,
        progress=45
    ))

    time.sleep(10)

    pprint(Trakt['scrobble'].stop(
        show=show,
        episode=episode,
        progress=90
    ))



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
from trakt.objects import Movie
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies/items',
        callback=authenticated_response('fixtures/users/me/lists/movies/items.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        items = Trakt['users/me/lists/movies'].items()

    # Ensure collection is valid
    assert_that(items, not_none())

    # Validate items
    assert_that(items, contains(
        # Mad Max: Fury Road
        all_of(
            instance_of(Movie),
            has_properties({
                'pk': ('imdb', 'tt1392190'),
                'title': 'Mad Max: Fury Road',
                'year': 2015,

                # Timestamps
                'listed_at': datetime(2015, 6, 24, 5, 50, 21, tzinfo=tzutc()),

                # Keys
                'keys': [
                    ('imdb', 'tt1392190'),
                    ('tmdb', '76341'),
                    ('slug', 'mad-max-fury-road-2015'),
                    ('trakt', '56360')
                ]
            })
        ),

        # Maggie (2015)
        all_of(
            instance_of(Movie),
            has_properties({
                'pk': ('imdb', 'tt1881002'),
                'title': 'Maggie',
                'year': 2015,

                # Timestamps
                'listed_at': datetime(2015, 6, 24, 5, 50, 21, tzinfo=tzutc()),

                # Keys
                'keys': [
                    ('imdb', 'tt1881002'),
                    ('tmdb', '287424'),
                    ('slug', 'maggie-1969'),
                    ('trakt', '184504')
                ]
            })
        )
    ))



from tests.core.helpers import authenticated_response

from trakt import Trakt
import responses


@responses.activate
def test_start():
    responses.add_callback(
        responses.POST, 'http://mock/scrobble/start',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    with Trakt.configuration.auth('mock', 'mock'):
        result = Trakt['scrobble'].start(
            movie={'ids': {'tmdb': 76341}}
        )

    assert result is not None


@responses.activate
def test_pause():
    responses.add_callback(
        responses.POST, 'http://mock/scrobble/pause',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    with Trakt.configuration.auth('mock', 'mock'):
        result = Trakt['scrobble'].pause(
            movie={'ids': {'tmdb': 76341}}
        )

    assert result is not None


@responses.activate
def test_stop():
    responses.add_callback(
        responses.POST, 'http://mock/scrobble/stop',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    with Trakt.configuration.auth('mock', 'mock'):
        result = Trakt['scrobble'].stop(
            movie={'ids': {'tmdb': 76341}}
        )

    assert result is not None



from tests.core.helpers import pagination_response

from hamcrest import *
from trakt import Trakt
from trakt.objects import Movie, Show, Season, Episode
import responses


@responses.activate
def test_basic():
    responses.add_callback(
        responses.GET, 'http://mock/sync/ratings',
        callback=pagination_response(
            'fixtures/sync/ratings.json',
            authenticated=True
        )
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        ratings = Trakt['sync/ratings'].get()

    # Ensure collection is valid
    assert_that(ratings, not_none())

    # Resolve all pages
    items = list(ratings)

    # Verify items
    assert_that(items, all_of(
        has_length(4),

        # TRON: Legacy (2010)
        has_item(all_of(
            instance_of(Movie),
            has_properties({
                'pk': ('imdb', 'tt1104001')
            })
        )),

        # Breaking Bad (2008)
        has_item(all_of(
            instance_of(Show),
            has_properties({
                'pk': ('tvdb', '81189')
            })
        )),

        # Breaking Bad (2008) - S00
        has_item(all_of(
            instance_of(Season),
            has_properties({
                'pk': 0
            })
        )),

        # Breaking Bad (2008) - S04E01
        has_item(all_of(
            instance_of(Episode),
            has_properties({
                'pk': (4, 1)
            })
        ))
    ))



from setuptools import setup, find_packages
import os
import re
import sys

BASE_DIR = os.path.dirname(__file__)

RE_DEPENDENCY_LINK = re.compile(r"^(?P<url>(?P<protocol>[\w\+]+):\/\/.*?)(?:#egg=(?P<egg>.*?))?$", re.IGNORECASE)
RE_EGG_VERSION = re.compile(r"^(?P<name>.*?)(?:-(?P<version>\d+\.\d+(?:\.\d+)?))?$", re.IGNORECASE)


def read_requirements_file(path):
    path = os.path.join(BASE_DIR, path)

    if not os.path.exists(path):
        sys.stderr.write('ERROR: Unable to read requirements file at %r\n', path)
        return [], []

    # Parse requirements from file
    dependency_links = []
    requirements = []

    for line in open(path):
        line = line.strip()

        if not line or line.startswith('#'):
            continue

        # Check for dependency links
        m_link = RE_DEPENDENCY_LINK.match(line)

        if m_link:
            egg = m_link.group('egg')

            if not egg:
                sys.stderr.write('ERROR: Missing "egg" parameter for: %r\n', line)
                continue

            # Add requirement
            m_egg = RE_EGG_VERSION.match(egg)

            if m_egg.group('name') and m_egg.group('version'):
                requirements.append('%s==%s' % (m_egg.group('name'), m_egg.group('version')))
            elif m_egg.group('name'):
                requirements.append(m_egg.group('name'))
            else:
                sys.stderr.write('ERROR: Invalid "egg" definition: %r\n', egg)
                continue

            # Add dependency link
            dependency_links.append(line)
            continue

        # Simple requirement
        requirements.append(line)

    return dependency_links, requirements


def read_requirements():
    dependency_links = []

    install_requires = []
    test_requires = []

    def parse(requires, path):
        d, r = read_requirements_file(path)

        dependency_links.extend(d)
        requires.extend(r)

    # Parse requirement files
    parse(install_requires, 'requirements.txt')
    parse(test_requires, 'requirements_test.txt')

    return {
        'dependency_links': dependency_links,

        'install_requires': install_requires,
        'tests_require': test_requires
    }

# Read current package version
version = {}

with open(os.path.join(BASE_DIR, "trakt", "version.py")) as fp:
    exec(fp.read(), version)

# Read contents of the "README.rst" file
with open(os.path.join(BASE_DIR, 'README.rst')) as fp:
    readme = fp.read()

# Setup configuration
setup(
    name='trakt.py',
    version=version['__version__'],
    description='Python interface for the trakt.tv API',
    long_description=readme,
    url='https://github.com/fuzeman/trakt.py',
    license='MIT',

    author='Dean Gardiner',
    author_email='me@dgardiner.net',

    # Packages
    platforms='any',
    packages=find_packages(include=[
        'trakt*'
    ]),

    # Classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],

    # Requirements
    setup_requires=[
        'pytest-runner'
    ],

    **read_requirements()
)



import logging
logging.basicConfig(level=logging.DEBUG)

from threading import Condition
from trakt import Trakt
import os


class Application(object):
    def __init__(self):
        self.is_authenticating = Condition()

        self.authorization = None

        # Bind trakt events
        Trakt.on('oauth.token_refreshed', self.on_token_refreshed)

    def authenticate(self):
        if not self.is_authenticating.acquire(blocking=False):
            print 'Authentication has already been started'
            return False

        # Request new device code
        code = Trakt['oauth/device'].code()

        print 'Enter the code "%s" at %s to authenticate your account' % (
            code.get('user_code'),
            code.get('verification_url')
        )

        # Construct device authentication poller
        poller = Trakt['oauth/device'].poll(**code)\
            .on('aborted', self.on_aborted)\
            .on('authenticated', self.on_authenticated)\
            .on('expired', self.on_expired)\
            .on('poll', self.on_poll)

        # Start polling for authentication token
        poller.start(daemon=False)

        # Wait for authentication to complete
        return self.is_authenticating.wait()

    def run(self):
        self.authenticate()

        if not self.authorization:
            print 'ERROR: Authentication required'
            exit(1)

        # Simulate expired token
        self.authorization['expires_in'] = 0

        # Test authenticated calls
        with Trakt.configuration.oauth.from_response(self.authorization):
            # Expired token, requests will return `None`
            print Trakt['sync/collection'].movies()

        with Trakt.configuration.oauth.from_response(self.authorization, refresh=True):
            # Expired token will be refreshed automatically (as `refresh=True`)
            print Trakt['sync/collection'].movies()

        with Trakt.configuration.oauth.from_response(self.authorization):
            # Current token is still valid
            print Trakt['sync/collection'].movies()

    def on_aborted(self):
        """Triggered when device authentication was aborted (either with `DeviceOAuthPoller.stop()`
           or via the "poll" event)"""

        print 'Authentication aborted'

        # Authentication aborted
        self.is_authenticating.acquire()
        self.is_authenticating.notify_all()
        self.is_authenticating.release()

    def on_authenticated(self, authorization):
        """Triggered when device authentication has been completed

        :param authorization: Authentication token details
        :type authorization: dict
        """

        # Acquire condition
        self.is_authenticating.acquire()

        # Store authorization for future calls
        self.authorization = authorization

        print 'Authentication successful - authorization: %r' % self.authorization

        # Authentication complete
        self.is_authenticating.notify_all()
        self.is_authenticating.release()

    def on_expired(self):
        """Triggered when the device authentication code has expired"""

        print 'Authentication expired'

        # Authentication expired
        self.is_authenticating.acquire()
        self.is_authenticating.notify_all()
        self.is_authenticating.release()

    def on_poll(self, callback):
        """Triggered before each poll

        :param callback: Call with `True` to continue polling, or `False` to abort polling
        :type callback: func
        """

        # Continue polling
        callback(True)

    def on_token_refreshed(self, authorization):
        # OAuth token refreshed, store authorization for future calls
        self.authorization = authorization

        print 'Token refreshed - authorization: %r' % self.authorization


if __name__ == '__main__':
    # Configure
    Trakt.base_url = 'http://api.staging.trakt.tv'

    Trakt.configuration.defaults.client(
        id=os.environ.get('CLIENT_ID'),
        secret=os.environ.get('CLIENT_SECRET')
    )

    app = Application()
    app.run()



from tests.core.helpers import read

from six.moves.urllib_parse import urlparse, parse_qsl
from trakt import Trakt
import responses


def search_callback(request):
    url = urlparse(request.url)
    query = dict(parse_qsl(url.query))

    if 'id' in query and 'id_type' in query:
        path = 'fixtures/search/lookup/%s/%s.json' % (
            query.get('id_type'),
            query.get('id')
        )
    else:
        path = 'fixtures/search/query/%s/%s/%s.json' % (
            query.get('type', 'all'),
            query.get('year', 'all'),
            query.get('query')
        )

    try:
        content = read(path)
        return 200, {}, content
    except:
        return 200, {}, '[]'


@responses.activate
def test_lookup_movie():
    responses.add_callback(
        responses.GET, 'http://mock/search',
        callback=search_callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    movie = Trakt['search'].lookup('tt0848228', 'imdb')

    assert movie.keys == [
        ('imdb', 'tt0848228'),
        ('tmdb', '24428'),
        ('slug', 'the-avengers-2012'),
        ('trakt', '14701')
    ]

    assert movie.title == "The Avengers"
    assert movie.year == 2012

    assert sorted(movie.images.keys()) == ['fanart', 'poster']
    assert movie.overview is not None
    assert movie.score is None


@responses.activate
def test_lookup_show():
    responses.add_callback(
        responses.GET, 'http://mock/search',
        callback=search_callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    show = Trakt['search'].lookup('tt0903747', 'imdb')

    assert show.keys == [
        ('tvdb', '81189'),
        ('tmdb', '1396'),
        ('imdb', 'tt0903747'),
        ('tvrage', '18164'),
        ('slug', 'breaking-bad'),
        ('trakt', '1388')
    ]

    assert show.title == "Breaking Bad"
    assert show.year == 2008

    assert sorted(show.images.keys()) == ['fanart', 'poster']
    assert show.overview is not None
    assert show.score is None


@responses.activate
def test_lookup_episode():
    responses.add_callback(
        responses.GET, 'http://mock/search',
        callback=search_callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    episode = Trakt['search'].lookup('tt0959621', 'imdb')

    assert episode.keys == [
        (1, 1),
        ('tvdb', '349232'),
        ('tmdb', '62085'),
        ('imdb', 'tt0959621'),
        ('tvrage', '637041'),
        ('trakt', '73482')
    ]

    assert episode.title == "Pilot"

    assert sorted(episode.images.keys()) == ['screenshot']
    assert episode.overview is not None
    assert episode.score is None

    assert episode.show.keys == [
        ('slug', 'breaking-bad'),
        ('trakt', '1388')
    ]

    assert episode.show.title == "Breaking Bad"
    assert episode.show.year == 2008

    assert sorted(episode.show.images.keys()) == ['fanart', 'poster']


@responses.activate
def test_query_movie():
    responses.add_callback(
        responses.GET, 'http://mock/search',
        callback=search_callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    movies = Trakt['search'].query('The Avengers', 'movie')

    assert [(m.score, (m.title, m.year)) for m in movies] == [
        (77.052734, ('Avenged', None)),
        (77.0176,   ('Avenger', 2006)),
        (60.26589,  ('The Avengers', 2012)),
        (60.26589,  ('The Avengers', 1998)),
        (60.26589,  ('The Avenger', 1960)),
        (60.26589,  ('The Avenger', 1931)),
        (60.26589,  ('The Avenging', 1982)),
        (60.26589,  ('The Avenger', 1947)),
        (55.793285, ('Invisible Avenger', 1954)),
        (55.652233, ('Crippled Avengers', 1978))
    ]


@responses.activate
def test_query_show():
    responses.add_callback(
        responses.GET, 'http://mock/search',
        callback=search_callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    shows = Trakt['search'].query('Breaking Bad', 'show')

    assert [(s.score, (s.title, s.year)) for s in shows] == [
        (54.809517,     ('Breaking Bad', 2008)),
        (28.975079,     ('Breaking Boston', 2014)),
        (28.89469,      ('Breaking In', 2011)),
        (26.28082,      ('Talking Bad', 2013)),
        (20.346865,     ('Donal MacIntyre: Breaking Crime', 2015)),
        (18.345793,     ('Good Times, Bad Times', 1990)),
        (1.1290063,     ('What About Brian', 2006)),
        (0.39297247,    ('It Could Be Worse', 2013)),
        (0.39297247,    ('Murder Police', None)),
        (0.3438509,     ('Pinocchio', 2014))
    ]


@responses.activate
def test_query_episode():
    responses.add_callback(
        responses.GET, 'http://mock/search',
        callback=search_callback,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    episodes = Trakt['search'].query('Breaking Bad', 'episode')

    assert [(e.score, (e.pk, e.title), (e.show.title, e.show.year)) for e in episodes] == [
        (77.16374, ((2, 13),    'Bad Breaks'),     ('Burn Notice', 2007)),
        (77.16374, ((1, 1),     'Breaking Bad'),   ("The Writers' Room", 2013)),
        (77.16374, ((2, 18),    'Breaking Bad'),   ('Honest Trailers', 2012)),
        (77.16374, ((4, 7),     'Bad Break'),      ('Bad Girls Club', 2006)),
        (77.16374, ((1, 8),     'Bad Break'),      ('Miami Ink', 2005)),
        (77.16374, ((2, 6),     'Breaking Bad'),   ('Pawn Stars UK', 2013)),
        (77.16374, ((6, 16),    'Bad Breaks'),     ('Trapper John, M.D.', 1979)),
        (77.16374, ((3, 8),     'Breaking Bad'),   ('Bad Days', 1969)),
        (77.16374, ((1, 261),   'Breaking Bad'),   ('The Totally Rad Show', 2007)),
        (77.16374, ((1, 2),     ' Breaking Bad'),  ('Fight Factory', 2012))
    ]



from tests.core.helpers import read

from trakt import Trakt
import responses


@responses.activate
def test_shows_updates():
    responses.add(
        responses.GET, 'http://mock/shows/updates',
        body=read('fixtures/shows/updates.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    response = Trakt.http.get('/shows/updates')
    data = response.json()

    assert data[0]['show']['title'] == 'Breaking Bad'
    assert data[1]['show']['title'] == 'The Walking Dead'


@responses.activate
def test_show_progress_collection():
    responses.add(
        responses.GET, 'http://mock/shows/tt0944947/progress/collection',
        body=read('fixtures/shows/tt0944947/progress/collection.json'), status=200,
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    response = Trakt.http.get('/shows/tt0944947/progress/collection')
    data = response.json()

    assert data['aired'] == 10
    assert data['completed'] == 6



from tests.core.helpers import assert_url

from trakt import Trakt
import pytest


def test_url():
    with Trakt.configuration.app(id=1234):
        assert_url(Trakt['oauth/pin'].url(), '/pin/1234')

    with pytest.raises(ValueError):
        Trakt['oauth/pin'].url()



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
import responses


@responses.activate
def test_movies():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies',
        callback=authenticated_response('fixtures/users/me/lists/movies.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        value = Trakt['users/me/lists/movies'].get()

    # Validate movies list
    assert_that(value, has_properties({
        'name': 'Movies',
        'description': None,
        'likes': 0,
    
        'allow_comments': True,
        'display_numbers': False,
    
        'updated_at': datetime(2015, 6, 22, 2, 25, tzinfo=tzutc()),
    
        'comment_count': 0,
        'item_count': 2,
    
        'privacy': 'private'
    }))


@responses.activate
def test_shows():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/shows',
        callback=authenticated_response('fixtures/users/me/lists/shows.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        value = Trakt['users/me/lists/shows'].get()

    # Validate shows list
    assert_that(value, has_properties({
        'name': 'Shows',
        'description': None,
        'likes': 0,

        'allow_comments': True,
        'display_numbers': False,

        'updated_at': datetime(2015, 6, 22, 2, 25, tzinfo=tzutc()),

        'comment_count': 0,
        'item_count': 3,

        'privacy': 'private'
    }))



from tests.core.helpers import authenticated_response

from trakt import Trakt
import responses


@responses.activate
def test_list_add():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies',
        callback=authenticated_response('fixtures/users/me/lists/movies.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.POST, 'http://mock/users/me/lists/123456/items',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        movies_list = Trakt['users/me/lists/movies'].get()

        result = movies_list.add({
            'shows': [
                {'ids': {'tvdb': 121361}}
            ]
        })

    assert result is not None


@responses.activate
def test_list_delete():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies',
        callback=authenticated_response('fixtures/users/me/lists/movies.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.DELETE, 'http://mock/users/me/lists/123456',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        movies_list = Trakt['users/me/lists/movies'].get()

        success = movies_list.delete()

    assert success is True


@responses.activate
def test_list_update():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies',
        callback=authenticated_response('fixtures/users/me/lists/movies.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.PUT, 'http://mock/users/me/lists/123456',
        callback=authenticated_response('fixtures/users/me/lists/shows.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        movies_list = Trakt['users/me/lists/movies'].get()

        result = movies_list.update(
            name="Shows (2)"
        )

    assert result is not None


@responses.activate
def test_list_remove():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies',
        callback=authenticated_response('fixtures/users/me/lists/movies.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.POST, 'http://mock/users/me/lists/123456/items/remove',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        movies_list = Trakt['users/me/lists/movies'].get()

        result = movies_list.remove({
            'shows': [
                {'ids': {'tvdb': 121361}}
            ]
        })

    assert result is not None


@responses.activate
def test_list_like():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies',
        callback=authenticated_response('fixtures/users/me/lists/movies.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.POST, 'http://mock/users/me/lists/123456/like',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        movies_list = Trakt['users/me/lists/movies'].get()

        success = movies_list.like()

    assert success is True


@responses.activate
def test_list_unlike():
    responses.add_callback(
        responses.GET, 'http://mock/users/me/lists/movies',
        callback=authenticated_response('fixtures/users/me/lists/movies.json'),
        content_type='application/json'
    )

    responses.add_callback(
        responses.DELETE, 'http://mock/users/me/lists/123456/like',
        callback=authenticated_response(data='{"mock": "mock"}'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        movies_list = Trakt['users/me/lists/movies'].get()

        success = movies_list.unlike()

    assert success is True


from trakt.core.helpers import to_iso8601, deprecated
from trakt.objects.core.helpers import update_attributes
from trakt.objects.video import Video


class Movie(Video):
    def __init__(self, client, keys, index=None):
        super(Movie, self).__init__(client, keys, index)

        self.title = None
        """
        :type: :class:`~python:str`

        Movie title
        """

        self.year = None
        """
        :type: :class:`~python:int`

        Movie year
        """

        self.watchers = None  # trending
        """
        :type: :class:`~python:int`

        Number of active watchers (returned by the :code:`Trakt['movies'].trending()`
        and :code:`Trakt['shows'].trending()` methods)
        """

    def to_identifier(self):
        """Returns the movie identifier which is compatible with requests that require
        movie definitions.

        :return: Movie identifier/definition
        :rtype: :class:`~python:dict`
        """

        return {
            'ids': dict(self.keys),
            'title': self.title,
            'year': self.year
        }

    @deprecated('Movie.to_info() has been moved to Movie.to_dict()')
    def to_info(self):
        """**Deprecated:** use the :code:`to_dict()` method instead"""
        return self.to_dict()

    def to_dict(self):
        """Dump movie to a dictionary

        :return: Movie dictionary
        :rtype: :class:`~python:dict`
        """

        result = self.to_identifier()

        result.update({
            'watched': 1 if self.is_watched else 0,
            'collected': 1 if self.is_collected else 0,

            'plays': self.plays if self.plays is not None else 0,
            'in_watchlist': self.in_watchlist if self.in_watchlist is not None else 0,
            'progress': self.progress,

            'last_watched_at': to_iso8601(self.last_watched_at),
            'collected_at': to_iso8601(self.collected_at),
            'paused_at': to_iso8601(self.paused_at)
        })

        if self.rating:
            result['rating'] = self.rating.value
            result['rated_at'] = to_iso8601(self.rating.timestamp)

        return result

    def _update(self, info=None, **kwargs):
        super(Movie, self)._update(info, **kwargs)

        update_attributes(self, info, [
            'title',
            'watchers'  # trending
        ])

        if info.get('year'):
            self.year = int(info['year'])

    @classmethod
    def _construct(cls, client, keys, info, index=None, **kwargs):
        movie = cls(client, keys, index=index)
        movie._update(info, **kwargs)

        return movie

    def __repr__(self):
        return '<Movie %r (%s)>' % (self.title, self.year)



from tests.core.helpers import authenticated_response

from datetime import datetime
from dateutil.tz import tzutc
from hamcrest import *
from trakt import Trakt
import responses


@responses.activate
def test_watched():
    responses.add_callback(
        responses.GET, 'http://mock/sync/watched/shows',
        callback=authenticated_response('fixtures/sync/watched/shows.json'),
        content_type='application/json'
    )

    Trakt.base_url = 'http://mock'

    with Trakt.configuration.auth('mock', 'mock'):
        collection = Trakt['sync/watched'].shows()

    # Ensure collection is valid
    assert_that(collection, not_none())
    assert_that(collection, has_length(9))

    # Chuck (2007)
    assert_that(collection[('tvdb', '80348')], has_properties({
        'pk': ('tvdb', '80348'),
        'title': 'Chuck',
        'year': 2007,

        # Seasons
        'seasons': all_of(
            has_length(1),
            has_entry(1, has_properties({
                # Episodes
                'episodes': all_of(
                    has_length(4),
                    has_entry(2, has_properties({
                        'pk': (1, 2),

                        'plays': 9,

                        # Timestamps
                        'last_watched_at': datetime(2015, 3, 10, 5, 21, 51, tzinfo=tzutc()),

                        # Flags
                        'is_watched': True
                    }))
                )
            }))
        ),

        # Keys
        'keys': [
            ('tvdb', '80348'),
            ('tmdb', '1404'),
            ('imdb', 'tt0934814'),
            ('tvrage', '15614'),
            ('slug', 'chuck'),
            ('trakt', '1395')
        ]
    }))



from trakt.core.context_collection import ListCollection, ContextCollection


def test_list_collection():
    m = ListCollection(
        lambda: [1, 2, 3],
        [4, 5, 6]
    )

    assert len(m) == 6

    assert 1 in m
    assert 10 not in m

    assert m[0] == 1
    assert m[-1] == 6

    m[4] = 9
    assert m[4] == 9


def test_context_collection():
    o = ContextCollection(
        ['base']
    )

    o.append('child-1')

    assert o.current == ['base', 'child-1']
    assert o[0] == 'base'
    assert o[-1] == 'child-1'

    o.append('child-2')
    assert o.current == ['base', 'child-1', 'child-2']
    assert o[-1] == 'child-2'

    o.pop()
    assert o.current == ['base', 'child-1']
    assert o[-1] == 'child-1'
